package com.yakindu.ea.examples.plugin.generators;

import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.runtime.CoreException;

import com.yakindu.bridges.ea.core.internal.utils.EAFileUtils;
import com.yakindu.bridges.ea.core.utils.URIToOSPathConverter;

public class HtmlTemplate {

	// some constants for json string escaping
	private static final Pattern CHARS_TO_ESCAPE = Pattern.compile("(\\\\|'|\b|\f|\n|\r|\t)",
			Pattern.DOTALL | Pattern.MULTILINE);
	private static final Map<String, String> CHARS_ESCAPE_MAP = new HashMap<String, String>() {
		private static final long serialVersionUID = 1L;
		{
			put("\\", "\\\\\\\\");
			put("'", "\\\\'");
			put("\b", "\\\\b");
			put("\f", "\\\\f");
			put("\n", "\\\\n");
			put("\r", "\\\\r");
			put("\t", "\\\\t");
		}
	};

	private String template;

	public HtmlTemplate(URL templateUrl) {
		this.template = loadHtmlTemplate(templateUrl);
	}

	public static String escapeJsonString(String string) {
		final Matcher matcher = CHARS_TO_ESCAPE.matcher(string);
		final StringBuffer result = new StringBuffer();
		while (matcher.find()) {
			final String illegalChar = matcher.group(1);
			final String escapedChar = CHARS_ESCAPE_MAP.get(illegalChar);
			matcher.appendReplacement(result, escapedChar);
		}
		matcher.appendTail(result);
		return result.toString();
	}

	public String prepareHtml(String data, ResourceInfo resourceInfo) {
		return template.replace("###title###", resourceInfo.resource.getURI().lastSegment()) //
				.replace("###eap-file###", URIToOSPathConverter.getFileFromURI(resourceInfo.resource.getURI())) //
				.replace("###generator###", getClass().getSimpleName()) //
				.replace("###depth###", String.valueOf(resourceInfo.getMaxDepth())) //
				.replace("###packages###", String.valueOf(resourceInfo.getPackages())) //
				.replace("###packagedElements###", String.valueOf(resourceInfo.getPackagedElements())) //
				.replace("###errors###", String.valueOf(resourceInfo.resource.getErrors().size())) //
				.replace("###warnings###", String.valueOf(resourceInfo.resource.getWarnings().size())) //
				.replace("###data###", data); // replace data last because it's the largest string
	}

	private static final String loadHtmlTemplate(URL templateUrl) {
		try {
			return EAFileUtils.loadFileContents(templateUrl.openStream());
		} catch (CoreException | IOException e) {
			throw new RuntimeException("Failed to load HTML template", e);
		}
	}
}
