package com.yakindu.ea.examples.orderingsoftware.template.jpa

import com.yakindu.ea.examples.orderingsoftware.template.core.PersistenceManagerTemplate
import com.yakindu.bridges.ea.examples.runtime.codegen.EACodegen
import com.yakindu.bridges.ea.examples.runtime.codegen.EACodegenFile
import org.eclipse.core.resources.IFile
import org.eclipse.uml2.uml.Package
import org.eclipse.xtend2.lib.StringConcatenation

class JPAPersistenceManagerTemplate extends PersistenceManagerTemplate {
	@EACodegenFile
	override path(Package element, IFile defaultFile) {
		return getTargetFilePath("jpa", "com/example/orderingsoftware/" + super.path(element, defaultFile).name)
	}
	
	@EACodegen("java")
	override generate(Package element) {
		super.generate(element)
	}
	
	override String generatePersistenceManager(Package element) '''
		«printPackageDeclaration(element)»

		import java.util.UUID;

		import javax.persistence.EntityManager;
		import javax.persistence.EntityManagerFactory;
		import javax.persistence.Persistence;

		«printClassComment»
		public class PersistenceManager implements AutoCloseable {
			private final EntityManagerFactory factory;

			private EntityManager entityManager; 

			public PersistenceManager() {
				factory = Persistence.createEntityManagerFactory("ShopDB");
			}

			public void beginTransaction() {
				if (entityManager == null) {
					entityManager = factory.createEntityManager();
					entityManager.getTransaction().begin();
				}
			}

			public boolean save(AbstractIDObject entity) {
				assert entityManager != null : "No transaction available.";
				if (entity != null) {
					if (entity.getId() == null) {
						entity.setId(UUID.randomUUID());
					}
					entityManager.persist(entity);
					return true;
				} else {
					return false;
				}
			}

			«FOR type : collectClasses(element) SEPARATOR StringConcatenation.DEFAULT_LINE_DELIMITER»
				public «type.name» get«type.name.toFirstUpper»(UUID id) {
					if (id != null) {
						return entityManager.find(«type.name».class, id);
					} else {
						return null;
					}
				}
			«ENDFOR»

			public void endTransaction() {
				if (entityManager != null) {
					entityManager.getTransaction().commit();
					entityManager.close();
					entityManager = null;
				}
			}

			@Override
			public void close() {
				endTransaction();
				factory.close();
			}
		}
	'''
}